import { select } from 'd3';

const network = {
  nodes: [
    {
      id: '192.168.162.127',
      name: 'MAIN',
      type: 'main',
      x: 20,
      y: 50,
      listeners: ['0.0.0.0:443']
    },
    {
      id: '192.168.162.128',
      name: 'SUB',
      type: 'sub',
      x: 40,
      y: 80,
      listeners: ['0.0.0.0:443', 
                  '0.0.0.0:8000']
    },
    {
      id: '192.168.162.129',
      name: 'PROX1',
      type: 'proxy',
      x: 60,
      y: 35,
      listeners: ['0.0.0.0:1337']
    },
    {
      id: '192.168.162.130',
      name: 'PROX2',
      type: 'proxy',
      x: 60,
      y: 50,
      listeners: ['0.0.0.0:1338']
    },
    {
      id: '192.168.162.131',
      name: 'PROX3',
      type: 'proxy',
      x: 60,
      y: 65,
      listeners: ['0.0.0.0:1339']
    },
    {
      id: '192.168.162.120',
      name: 'DEVICE123',
      type: 'agent',
      x: 80,
      y: 25,
      listeners: []
    },
    {
      id: '192.168.162.121',
      name: 'DEVICE124',
      type:'agent',
      x: 80,
      y: 50,
      listeners: []
    },
    {
      id: '192.168.162.122',
      name: 'DEVICE125',
      type: 'agent',
      x: 80,
      y: 75,
      listeners: ['0.0.0.0:8000']
    },
    {
      id: '192.168.162.123',
      name: 'DEVICE126',
      type: 'agent2',
      x: 90,
      y: 75,
      listeners: ['0.0.0.0:8000']
    },
  ],
  links: [
    // {
    //   source: '192.168.162.127',
    //   target: '192.168.162.128',
    //   port: 8000,
    //   type: 'websocket',
    // },
    {
      source: '192.168.162.129',
      target: '192.168.162.127',
      port: 443,
      type: 'https',
    },
    {
      source: '192.168.162.130',
      target: '192.168.162.127',
      port: 443,
      type: 'https',
    },
    {
      source: '192.168.162.131',
      target: '192.168.162.127',
      port: 443,
      type: 'https',
    },
    {
      source: '192.168.162.120',
      target: '192.168.162.129',
      port: 1337,
      type: 'callback',
    },
    {
      source: '192.168.162.121',
      target: '192.168.162.130',
      port: 1338,
      type: 'callback',
    },
    {
      source: '192.168.162.122',
      target: '192.168.162.131',
      port: 1339,
      type: 'callback',
    },
    {
      source: '192.168.162.122',
      target: '192.168.162.123',
      port: 8000,
      type: 'tunnel',
    },
    {
      source: '192.168.162.121',
      target: '192.168.162.122',
      port: 8000,
      type: 'reverse',
    },
  ],
};

const icon_type = {
	'main': 'fa-globe',
  'sub': 'fa-shield-alt',
  'proxy': 'fa-compass',
  'agent': 'fa-biohazard',
  'agent2': 'fa-laptop',
}

const nodeColor = "black";
const textColor = "white";
const lineWidth = 2;
const lineColor = "lightgrey";
const circleRadius = 20;
const forceStrength = 0.01;
const linkStrength = 0.01;

var svg = select("svg"),
    width = +svg.attr("width"),
    height = +svg.attr("height");

network.nodes.forEach(node => {
  node.x = (node.x * width) / 100;
  node.y = (node.y * height) / 100;
});
                            
network.links.forEach(link => {
  let source = network.nodes.find(node => node.id == link.source);
  let target = network.nodes.find(node => node.id == link.target);

  if (source.x < target.x) {
  	link['dir'] = 'forward';
  } else {
  	link['dir'] = 'reverse';
    let temp = link.source;
    link.source = link.target;
    link.target = temp;
  }
});


var link_template = d3.linkVertical()
    .source(function(d) {
      let node = network.nodes.find(node => node.id == d.source);
	    return [node.x, node.y];
    })
    .target(function(d) {
      let node = network.nodes.find(node => node.id == d.target);
      return [node.x, node.y];
    });

var tooltip = select("body")
    .append("div")
    .style("position", "absolute")
    .style("z-index", "10")
    .style("visibility", "hidden")
    .style("background", "#fff")
    .text("a simple tooltip");

// build the arrow.
// svg.append("svg:defs").selectAll("marker")
//     .data(["end"])      // Different link/path types can be defined here
//   .enter().append("svg:marker")    // This section adds in the arrows
//     .attr("id", String)
//     .attr("viewBox", "0 -5 10 10")
//     .attr("refX", 25)
//     .attr("refY", 0)
//     .attr("markerWidth", 6)
//     .attr("markerHeight", 6)
//     .attr("orient", "auto")
//   .append("svg:path")
//     .attr("d", "M0,-5L10,0L0,5");

var link = svg.selectAll(".linePath")
    .data(network.links)
    .join("path")
    .attr("d", link_template)
		.attr("class", "linePath")
		.attr("id", function(d, i) { return "linePath_"+i; })
		.attr("stroke-width", lineWidth)
    .attr("stroke", lineColor)
    .attr("fill", "none")
		.classed("link", true); 

//   .enter().append("svg:path")
// //    .attr("class", function(d) { return "link " + d.type; })
//     .attr("class", "link")
//     .attr("marker-end", "url(#end)");

var linkText = svg.selectAll(".lineText")
    .data(network.links)
    .enter().append("text")
		.attr("class", "lineText")
    .attr("fill", lineColor)
		.attr('dy', -4)
    .append("textPath")
		.attr("xlink:href", function(d,i) { return "#linePath_"+i; })
    .style("text-anchor", d => d.dir == 'forward' ? "end" : "start" )
		.attr("startOffset", d => d.dir == 'forward' ? "85%" : "45px" )
		.text(function(d) {return "tcp:"+d.port; });

var node = svg.append("g")
    	.attr("class", "nodes")
    .selectAll("g")
    .data(network.nodes)
    .enter().append("g")
    .on("mouseover", function(event, d){
      console.log('in mouseover: ' + d.name );
      var message = `<dl><dt>${d.name}</dt>`
      d.listeners.forEach(listener => {
	      message += `<dt>${listener}</dt>`    	
      });
			message += '</dl>'
      tooltip.html(message); 
      return tooltip.style("visibility", "visible");})
    .on("mousemove", function(event){
      console.log('in mousemove');
      return tooltip.style("top", (event.pageY-10)+"px").style("left",(event.pageX+10)+"px");})
    .on("mouseout", function(){
      console.log('in mouseout');
      return tooltip.style("visibility", "hidden");})
		.call(d3.drag().on("drag", dragged));
		// .on("mouseover", function(event) {
		// console.log('in mouseover');
		// div.transition()		
		// .duration(200)		
		// .style("opacity", .9);		
		// div	.html("<div>Test</div>")
		// .style("left", (event.pageX) + "px")		
		// .style("top", (event.pageY - 28) + "px");	
		// })					
		// .on("mouseout", function(d) {		
		// console.log('in mouseout');
		// div.transition()		
		// .duration(500)		
		// .style("opacity", 0);	
		// });

// var color = d3.scaleOrdinal(d3.schemeCategory20);

var circleG = node.append("g")

var circle = circleG.append("circle")
    .attr("r", circleRadius)
    .attr("fill", nodeColor)

var image = circleG.append('svg:foreignObject')
    .attr('height', `${circleRadius*2}px`)
    .attr('width', `${circleRadius*2}px`)
    .attr('color' , 'white')
    .attr('font-size' , `${circleRadius}px`)
		.attr('transform', `translate(${-circleRadius}, ${-circleRadius*3/5})`)
    .html(function(d) { return `<i class='fa ${icon_type[d.type]}'></i>`} );

var labels = circleG.append("text")
    .text(function(d) {
      return d.name;
    })
    .attr('fill' , "white")
		.attr("text-anchor", "middle")
		.attr('y', -circleRadius-5);

var div = svg.append("div")	
    .attr("class", "tooltip")				
    .style("opacity", 0);

var simulation = d3.forceSimulation()
    .force("link", d3.forceLink().id(function(d) { return d.id; }))
    .force("charge", d3.forceManyBody())
    .force("center", d3.forceCenter(width / 2, height / 2)
    .strength(forceStrength));

simulation
    .nodes(network.nodes)
    .on("tick", ticked);

simulation.force("link")
    .links(network.links)
		.strength(linkStrength);

function dragged(event, d) {
  d.x = event.x; d.y = event.y;
  d3.select(this).attr("cx", event.x).attr("cy", event.y);
  render();
}

function ticked() {
  render();
}

function render() {
  link.attr("d", d3.linkHorizontal()
      .x(function(d) { return d.x; })
      .y(function(d) { return d.y; }));

  node
      .attr("transform", function(d) {
        return "translate(" + d.x + "," + d.y + ")";
      })
}
